*  FILE NAME: FileMgr.prg
*  BY: (John Graceland)
*  DATE: 01/01/94
*  DESC: .DBF & NTX manager
*  Compiler: CA-Clipper v5.01

#include "inkey.ch"
#include "setcurs.ch"
#define DOUBLE_BOX  ("ͻȺ ")

*************
FUNCTION Main
*************
// file manager demo

/* reindex all files - whenever you need to perform reindexing, make a call to
manualIndex().  Displays a directory listing of all database files.  
Features include, Tag multiple files, search for a file (type file name)
*/
ManualIndex()

/* open a file - whenever you want to open a file, make a call to sys_open()
Opens dbf & ntx files that are currently not open or selects the appropriate
workarea is the file is open.  Also, if the index file does not exist, it
creates it on the fly.
*/

// use debugger to see the results 
Sys_Open("Cus")
Sys_Open("Rep")
Sys_Open("Ar")

// Select an open file
Sys_Open("Cus")

// Close an open file
Sys_Close("Cus")

// using my alert()
Alert("Normal Shutdown",,,"W+/BG,B/W+")

**************************************************
FUNCTION Sys_Open( cFileName,lExclUse,lManualNtx )
**************************************************
/* 
* file opening routine.  Call this function whenever you want to open 
* or select a file.  It will manage it for you.
* syntax: Sys_Open( <cFileName>, [<lExclUse>, lManualIndex>] )
*/

LOCAL;
lRetval := (.F.),;
cFullname

// parse the file extension if included
cFileName := IF( AT(".",cFileName) != 0 ,;
             SUBSTR( cFileName,1, AT(".", cFileName)-1),cFIleName)

// test for exclusive use
lExclUse := IF( lExclUse == NIL, .F., lExclUse) 

// add suffix to file name prefix
cFullName := cFileName + ".dbf"

// disable manual index flag if argument was not passed
lManualNtx := IF( lManualNtx == NIL, .F., lManualNtx )

IF FILE( cFullName )

   // is file currently open in another work area
   IF SELECT( cFileName ) == 0  // not open
      // file is not open in any work area
      IF lExclUse
         USE (cFileName) EXCLUSIVE NEW
      ELSE
         USE (cFileName) SHARED NEW
      END
      lRetval := NETERR()
      IF !lRetval
         // open appropriate index files
         Sys_OpenIndex( lManualNtx )
      ELSE
         Alert("< Sharing Violation >  The file ("+ALLTRIM(cFullName)+") cannot be opened at this time.",{"Quit"},,,"W+/BG,B/W+")
      END
   ELSE
      // file is already open, select work area
      lRetval := (.T.)
      SELECT ( cFileName )
   END
ELSE
   ALERT("< Open Error >  The file ("+ALLTRIM(cFullName)+") not in path.",{"Quit"})
END

RETURN ( !lRetval )

********************
FUNCTION ManualIndex
********************
/*
Manual indexing/reindexing routine
Call this routine whenever you want to reindex
*/

LOCAL;
nChoice,;
aFileData[ ADIR("*.DBF") ],;
cDbfHandle,nX,bf10,;
cScreen := SAVESCREEN(0,0,MAXROW(),MAXCOL() ),;
aPicks  := {}

SETCOLOR("W+/W,W/N")
SCROLL()
ADIR("*.DBF",aFileData)
ASORT( aFileData )

@ 00,00 SAY PADC("Database Reindexing",80," ") COLOR IF(ISCOLOR(),"W+/BG","N/W")
@ 22,00 SAY PADC("F10 Pack: Disabled",80," ") COLOR IF(ISCOLOR(),"W+/BG","N/W")
@ 23,00 SAY PADC("Enter to Select    <F5> Tag/Untag    <Esc> Exit",80," ") COLOR IF(ISCOLOR(),"W+/BG","N/W")
@ 24,00 SAY PADC("Search: Type a file name",80," ") COLOR IF(ISCOLOR(),"W+/BG","N/W")

CLOSE ALL
DO WHILE LASTKEY() # K_ESC
   aPicks := aPickList( 03,32,18,45,"  Directory", @aFileData, @aPicks )

   IF LEN(aPicks) != 0
      FOR n:=1 TO LEN(aPicks)
         cDbfHandle := SUBSTR(aFileData[aPicks[n]],1, AT(".",aFileData[aPicks[n]])-1)
         Sys_Open( cDbfHandle,.T.,.T.)
         Sys_Close( cDbfHandle ) 
         IF INKEY() == K_ESC 
            EXIT
         ENDIF
      NEXT
      aPicks := {}
   END

END

RESTSCREEN(0,0,MAXROW(),MAXCOL(), cScreen )
RETURN ( NIL )

**********************************
STATIC FUNCTION TogglePack( cSet )
**********************************
/*
a Set/get routine, used to enable/disable and query pack routine
*/

STATIC lPacko := (.F.)
IF cSet == NIL
   lPacko := !lPacko
   @ 22,0 SAY PADC("F10 Pack: "+IIF(lPacko,"Enabled ","Disabled"),80," ") COLOR IF(ISCOLOR(),"W+/BG","N/W")
ENDIF

RETURN lPacko

************************************
FUNCTION Sys_OpenIndex( lManualNtx )
************************************
/*
Indexing routines
   -test for existance of index file
   -create index on non-existance
   -open index file set
add your indexing code here for each .dbf
*/

LOCAL ;
cNtxPreFix,;
cNtxFile,;
nFileCount,;
cScreen
lManualNtx:=IIF(lManualNtx==NIL,.F.,lManualNtx)

IF lManualNtx.AND.TogglePack("Query")
   cScreen := SAVESCREEN( MAXROW(),0,MAXROW(),MAXCOL() )
   @ MAXROW(),0 SAY PADC("Packing (Purging Deleted Records)",80," ")
   SET INDEX TO
   PACK
   INKEY(2)
   RESTSCREEN( MAXROW(),0,MAXROW(),MAXCOL(), cScreen  )
ENDIF

IF ALIAS() == "CUS"
   FOR nFileCount := 1 TO 3
      cNtxPreFix := "CUS" + ALLTRIM( STR( nFileCount ) )
      cNtxFile := cNtxPreFix+".ntx"
      IF !FILE( cNtxFile ) .OR. lManualNtx
         Sys_Reindex( cNtxPrefix )
      END
   NEXT
   SET INDEX TO Cus1,Cus2,Cus3

ELSEIF ALIAS() == "REP"
   FOR nFileCount := 1 TO 3
      cNtxPreFix := "REP" + ALLTRIM( STR( nFileCount ) )
      cNtxFile := cNtxPreFix+".ntx"
      IF !FILE( cNtxFile ) .OR. lManualNtx
         Sys_Reindex( cNtxPrefix )
      END
   NEXT
   SET INDEX TO Rep1,Rep2,Rep3

ELSEIF ALIAS() == "AR"
   FOR nFileCount := 1 TO 3
      cNtxPreFix := "AR" + ALLTRIM( STR( nFileCount ) )
      cNtxFile := cNtxPreFix+".ntx"
      IF !FILE( cNtxFile ) .OR. lManualNtx
         Sys_Reindex( cNtxPrefix )
      END
   NEXT
   SET INDEX TO Ar1,Ar2,Ar3

// add additional index open routines as needed

END

RETURN ( NIL )

***************************
FUNCTION Sys_Reindex(cFile)
***************************
LOCAL cScreen:=SAVESCREEN( MAXROW(),0,MAXROW(),MAXCOL() )

@ 24,0 SAY PADC( "Indexing "+ALLTRIM(cFile)+"  Please wait...",80," ") COLOR IF(ISCOLOR(),"W+/R","W+/N")

IF UPPER(cFile)=="CUS1"
   INDEX ON UPPER(Cus->cus_id) TO Cus1
ELSEIF UPPER(cFile)=="CUS2"
   INDEX ON UPPER(Cus->company) TO Cus2
ELSEIF UPPER(cFile)=="CUS3"
   // used to recycle deleted records  (details in another episode)
   INDEX ON !DELETED() TO Cus3

ELSEIF UPPER(cFile)=="REP1"
   INDEX ON UPPER(Rep->rep_id) TO Rep1
ELSEIF UPPER(cFile)=="REP2"            
   INDEX ON UPPER(Rep->last_name) TO Rep2
ELSEIF UPPER(cFile)=="REP3"
   INDEX ON !DELETED() TO Rep3

ELSEIF UPPER(cFile)=="AR1"
   INDEX ON UPPER(Ar->cus_id + Ar->inv_no) TO Ar1
ELSEIF UPPER(cFile)=="AR2"            
   INDEX ON UPPER(Ar->cus_id + DTOS(Ar->date) + Ar->inv_no ) TO Ar2
ELSEIF UPPER(cFile)=="AR3"
   INDEX ON !DELETED() TO Ar3

// add additional index routines as needed

END

RESTSCREEN( MAXROW(), 0, MAXROW(), MAXCOL(), cScreen )
RETURN (NIL)

*******************************
FUNCTION Sys_Close( cFileName )
*******************************
LOCAL nArea := SELECT()
// parse the file extension if included
cFileName := IF( AT(".",cFileName) != 0 ,;
             SUBSTR( cFileName,1, AT(".", cFileName)-1),cFIleName)

IF SELECT( cFileName ) != 0
   SELECT ( cFileName )
   USE
   SELECT( nArea )
ENDIF

RETURN (NIL)

*****************
FUNCTION Sys_Quit
*****************
SCROLL()
CLOSE ALL
@ 0,0 SAY "Normal Shutdown..."
QUIT

#xcommand      STABILIZE <obj> INKEY [TO] <x>  =>;
               DISPBEGIN();;
               WHILE (!<obj>:stabilize()) .AND.;
                    ((<x> := INKEY()) == 0);;
               END;;
               DISPEND()

***************************************************************
FUNCTION APickList(nTop,nLeft,nBottom,nRight,cTitle,aList,aTag)
***************************************************************
LOCAL b, nKey, column
LOCAL nSubscript:=1, k:=0,;
      cScreen   :=SAVESCREEN(00,00,24,79),;
      cOldColor :=SETCOLOR()
LOCAL cOldCrsr  :=SETCURSOR(0)
LOCAL lRetval   := "",a_Tag:={}
LOCAL lTag      := IF(aTag == NIL,.F.,.T.)
STATIC cBuffer  :=""
aTag            :=IF(aTag == NIL,{},aTag)

nBottom := IF(nBottom>=23,22,nBottom)

@ nTop-1,nLeft-1,nBottom+1,nRight+1 BOX DOUBLE_BOX

// TBrowse object for values
b               := TBrowseNew( nTop, nLeft, nBottom, nRight )
b:headSep       := ""
b:skipBlock     := {|x| NavArray(x, @nSubscript,LEN(aList))}
b:goTopBlock    := {|| nSubscript := 1}
b:goBottomBlock := {|| nSubscript := LEN(aList)}
b:colorSpec     := "W+/BG, N/W, R*/W, N/Bg"

column := TBColumnNew(PAD(cTitle,nRight-nLeft," "),{|| Tagged(@aTag,nSubScript)+""+PADR(aList[nSubscript],12," ")   })
b:addColumn( column )
column:colorBlock := { || IIF(IsTagged(@aTag,nSubScript),{4,3},{1,2}) }

WHILE .T.
   STABILIZE b INKEY TO nKey

   IF (b:stable)
      nKey := INKEY(0)
   ENDIF

   IF !TbMoveCursor( nKey, b )
      IF nKey == K_ESC
         EXIT
      ELSEIF nKey == K_ENTER .AND. lTag
         IF LEN(aTag) == 0
            AADD( aTag, nSubScript )
         END
         EXIT
      ELSEIF nKey == K_ENTER
         lRetval:=aList[nSubScript]
         //KEYBOARD CHR(K_ENTER)
         EXIT
      ELSEIF nKey == K_F5
         Tag( @aTag, nSubScript )
         b:RefreshAll()
      ELSEIF nKey == K_F10  
         TogglePack()
      ELSE
         // search
         cBuffer+=CHR(nKey)
         n := ASCAN( aList, {|x| UPPER(x) = UPPER(cBuffer)}) 
         IF n != 0
            b:GoTop()
            nSubScript := n
            b:RefreshAll()
         ELSE
           TONE(500,3)
           cBuffer:=""
         END
      END
   ELSE
      cBuffer:=""
   END
END

SETCURSOR(cOldCrsr)
SETCOLOR(cOldColor)
//RESTSCREEN(00,00,24,79,cScreen)
IF lTag
   RETURN aTag
ELSE
   RETURN lRetval
END

************************************
FUNCTION Tagged( aTag, nSubScript )
************************************
RETURN IIF( ASCAN(aTag,nSubScript ) == 0  , " ","")

****************************************
STATIC FUNCTION Tag( aTag, nSubscript )
****************************************
LOCAL n:=ASCAN( aTag, nSubScript ) 
IF n != 0
   ADEL(aTag, n )
   ASIZE(aTag,LEN(atag)-1)
ELSE
   AADD( aTag, nSubScript )
END
RETURN (NIL)

************************************
FUNCTION IsTagged( aTag, nSubScript )
************************************
RETURN IIF( ASCAN(aTag,nSubScript ) == 0  ,.F.,.T.)

***************************************************
STATIC FUNCTION NavArray(nRequest, nSubscript, nALen)
***************************************************
LOCAL k := 0

IF nRequest == 0
   k := 0

ELSEIF nRequest > 0
   IF nRequest < (nALen - nSubscript)
      k := nRequest
   ELSE
      k := nALen - nSubscript
   ENDIF

ELSEIF nRequest < 0
   // Skip back
   IF nRequest < (1 - nSubscript)
      k := 1 - nSubscript

   ELSE
      k := nRequest

   ENDIF

ENDIF

// Reposition
nSubscript += k
RETURN (k)

******************************************
STATIC FUNCTION TBMoveCursor( nKey, oObj )
******************************************
LOCAL nFound
STATIC aKeys := ;
    { K_DOWN      , {|b| b:down()},;
      K_UP        , {|b| b:up()},;
      K_PGDN      , {|b| b:pageDown()},;
      K_PGUP      , {|b| b:pageUp()},;
      K_CTRL_PGUP , {|b| b:goTop()},;
      K_CTRL_PGDN , {|b| b:goBottom()},;
      K_RIGHT     , {|b| b:right()},;
      K_LEFT      , {|b| b:left()},;
      K_HOME      , {|b| b:home()},;
      K_END       , {|b| b:end()},;
      K_CTRL_LEFT , {|b| b:panLeft()},;
      K_CTRL_RIGHT, {|b| b:panRight()},;
      K_CTRL_HOME , {|b| b:panHome()},;
      K_CTRL_END  , {|b| b:panEnd()} }

nFound := ASCAN( aKeys, nKey )
IF (nFound != 0)
   EVAL( aKeys[++nFound], oObj )

ENDIF
RETURN (nFound != 0)

************************************************************
FUNCTION Alert( cMessage, aChoices, nTopRow, cColor, lBell )
************************************************************
/*
 - Alert()
 - John Graceland
 - Substitute for Clipper's ALERT() 
 - Clipper 5.n
 - Syntax: Alert( <cMessage> [,<aChoices>, <nTopRow>, <cColor>, <lBell>] )
 - Features: Emulates Clipper's ALERT(), additional options are
             Starting row position of box, set color, sound bell
             Envoke Clippers dbugger from within Alert()
 - Returns a numeric value equal to the option selected, <Esc> = 0
*/

LOCAL;
n,;
nCol,;
nRow,;
cScreen,;
nPromLeft,;
nBoxLeft,;
nBoxRight,;
nMsgLen,;
nBoxLen    := 0,;
nPromLen   := 0,;
cOldColor  := SETCOLOR(),;
cOldWrap   := SET( 35, "ON" ),;
lOldCenter := SET( 37, .F. ),;
lOldBlink  := SETBLINK(.F.),;
aMessage   := {},;
bPgUp      := SETKEY(K_PGUP, {||NIL} ),;
bPgDn      := SETKEY(K_PGDN, {||NIL} ),;
nChoice

IF aChoices==NIL
   // force "OK" if no choices are passed
   aChoices:={}
   AADD(aChoices,"OK")
END

// set length of prompt line
AEVAL( aChoices, {|aArray| nPromLen += LEN(aArray)+2 })

// load message array
WHILE ";" $ cMessage
   // test for multiple line message
   AADD(aMessage, SUBSTR(cMessage,1,AT(";",cMessage)-1 ) )
   cMessage := STUFF(cMessage,1,AT(";",cMessage), "" )
END
IF "" != cMessage
   // add remaining message string to array
   AADD(aMessage,cMessage)
END
nMsgLen := LEN(aMessage)

// determine box length based on largest message line
AEVAL( aMessage, {|aArray| nBoxLen := MAX(nBoxLen,LEN(aArray)+4) })

nPromLen  += ( (LEN(aChoices)+1) * 2 )
cColor    := IIF( ISCOLOR(), IIF( cColor == NIL,"W+/R,W+/B",cColor),"N/W,W+/N")
lBell     := IIF( lBell == NIL, .F., lBell )
nTopRow   := IIF( nTopRow == NIL, 10, nTopRow )

// test for the larger of box lenght vs. length of prompt line
nBoxLen   := IIF( nBoxLen > nPromLen, nBoxLen, nPromLen + 4 )

nBoxLeft  := ( MAXCOL() - nBoxLen ) / 2
nBoxRight := nBoxLeft + nBoxLen
nPromLeft := ( ( MAXCOL()-nPromLen ) / 2) 

// save the original screen
cScreen := SAVESCREEN(nTopRow, nBoxLeft, nTopRow+3+nMsgLen, nBoxRight+1 )
DISPBEGIN()

SETCOLOR( cColor )
// display Alert box
@ nTopRow, nBoxLeft, nTopRow+3+nMsgLen, nBoxRight+1 BOX "Ŀ " 

FOR n:=1 TO nMsgLen
   @ nTopRow+n,nBoxLeft+1 SAY PADC( aMessage[n], nBoxLen, SPACE(1) )
NEXT

nCol       := nPromLeft + 3 
cOldWrap   := SET( 35, "ON" )
lOldCenter := SET( 37, .F. )

// Display the prompt line
nRow:=ROW()+2
FOR n:=1 TO LEN( aChoices )
   @ nRow, nCol PROMPT SPACE(1)+aChoices[n]+SPACE(1)
   nCol += LEN(aChoices[n])+4 
NEXT
DISPEND()

IF lBell
   // sound bell
   TONE(499,2)
   TONE(299,1)
ENDIF
MENU TO nChoice

// reset stuff
SET( 35, cOldWrap )
SET( 37, lOldCenter )
SETBLINK( lOldBlink )
RESTSCREEN( nTopRow, nBoxLeft, nTopRow+3+nMsgLen, nBoxRight+1,cScreen )
SETCOLOR( cOldColor )
SETKEY(K_PGUP, bPgUp )
SETKEY(K_PGDN, bPgDn )

RETURN IF( (LASTKEY()=K_ESC.OR.LASTKEY()=K_PGUP.OR.LASTKEY()=K_PGDN),0,nChoice)







